// ASEC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "windows.h"
#include "bfm.h"

#define NUM_VERTEX			"*MESH_NUMVERTEX"
#define NUM_FACE			"*MESH_NUMFACES"
#define NUM_TVERTEX			"*MESH_NUMTVERTEX"
#define NUM_TFACE			"*MESH_NUMTVFACES"
#define VERTEX_LIST			"*MESH_VERTEX_LIST"
#define VERTEX				"*MESH_VERTEX"
#define FACE_LIST			"*MESH_FACE_LIST"
#define FACE				"*MESH_FACE"
#define NORMALS				"*MESH_NORMALS"
#define FACE_NORMAL			"*MESH_FACENORMAL"
#define NVERTEX				"*MESH_VERTEXNORMAL"
#define TVERTEX				"*MESH_TVERT"
#define TFACE				"*MESH_TFACE"
#define TEXTURE				"*BITMAP"
#define UTILE				"*UVW_U_TILING"
#define VTILE				"*UVW_V_TILING"
#define UOFFSET				"*UVW_U_OFFSET"
#define VOFFSET				"*UVW_V_OFFSET"
#define GEOMOBJECT			"*GEOMOBJECT"
#define NODENAME			"*NODE_NAME"
#define WIREFRAMECOLOR		"*WIREFRAME_COLOR"
#define GROUP				"*GROUP"
#define MAT_COUNT			"*MATERIAL_COUNT"
#define MAT_MATERIAL		"*MATERIAL"
#define MAT_REFERENCE		"*MATERIAL_REF"
#define MAT_SUBMATERIAL		"*SUBMATERIAL"
#define MAT_NUMSUBMTL		"*NUMSUBMTLS"
#define MAT_AMBIENT			"*MATERIAL_AMBIENT"
#define MAT_DIFFUSE			"*MATERIAL_DIFFUSE"
#define MAT_SPECULAR		"*MATERIAL_SPECULAR"
#define MESH_MTLID			"*MESH_MTLID"



#define SIG_MATERIAL_COUNT			(BFMINT8)1
#define SIG_MATERIAL				(BFMINT8)2
#define SIG_MATERIAL_AMBIENT		(BFMINT8)3
#define SIG_MATERIAL_DIFFUSE		(BFMINT8)4
#define SIG_MATERIAL_SPECULAR		(BFMINT8)5
#define SIG_TEXTURE_NAME			(BFMINT8)6
#define SIG_NODENAME				(BFMINT8)7
#define SIG_WIREFRAMECOLOR			(BFMINT8)8
#define SIG_TEXTURE_UTILE			(BFMINT8)9
#define SIG_TEXTURE_VTILE			(BFMINT8)10
#define SIG_TEXTURE_UOFFSET			(BFMINT8)11
#define SIG_TEXTURE_VOFFSET			(BFMINT8)12
#define SIG_GEOMOBJECT				(BFMINT8)13
#define SIG_NUM_VERTEX				(BFMINT8)14
#define SIG_NUM_FACE				(BFMINT8)15
#define SIG_NUM_TVERTEX				(BFMINT8)16
#define SIG_NUM_TFACE				(BFMINT8)17
#define SIG_VERTEX					(BFMINT8)18
#define SIG_FACE					(BFMINT8)19
#define SIG_TVERTEX					(BFMINT8)20
#define SIG_TFACE					(BFMINT8)21
#define SIG_MATERIAL_REF			(BFMINT8)22
#define SIG_SUBMATERIAL				(BFMINT8)23
#define SIG_NUMSUBMTL				(BFMINT8)24
#define SIG_GROUP					(BFMINT8)25
#define SIG_ENDGROUP				(BFMINT8)26


typedef struct
{
	BFMINT8	object_count;
	BFMINT8	mat_count;
	BFMINT16 face_count;
} FILE_HEADER;

FILE_HEADER	fh;


BFManager bfm_bin;


void aseParse(FILE *s)
{
	char data[255];

	// get file size
	fseek(s, 0, SEEK_END);
	LONG fs = ftell(s);

	printf("read file(%d bytes)...\n\n", fs);

	rewind(s);


	while(!feof(s))
	{
		fscanf(s, "%s", data);

		if (!strcmp(data, MAT_MATERIAL))
		{
			fh.mat_count++;
		}
		else if (!strcmp(data, GEOMOBJECT))
		{
			fh.object_count++;
		}
		else if (!strcmp(data, NODENAME))
		{
			char buf[MAX_BF_NAME];
			
			fscanf(s, " \"%s", buf);
			
			// to remove "
			buf[strlen(buf)-1] = NULL;

			bfm_bin.WriteSig(SIG_NODENAME);
			bfm_bin.WriteInt8((PBFMINT8)buf, MAX_BF_NAME);
			printf("object name: %s (ID: %d)\n", buf, fh.object_count);
		}
		else if (!strcmp(data, NUM_VERTEX))
		{
			short v;

			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_VERTEX);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			printf("* total vertices: %d\n", v);
		}
		else if (!strcmp(data, NUM_FACE))
		{
			short v;

			fscanf(s, "%d", &v);

			bfm_bin.WriteSig(SIG_NUM_FACE);
			bfm_bin.WriteInt16((PBFMINT16)&v, 1);
			printf("* total faces: %d\n", v);
		}
		// read wire-frame color
		else if (!strcmp(data, WIREFRAMECOLOR))
		{
			float f[3];
			
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);

			bfm_bin.WriteSig(SIG_WIREFRAMECOLOR);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
			printf("* wire frame color: %f, %f, %f\n", f[0], f[1], f[2]);
		}
		// read vertex
		else if (!strcmp(data, VERTEX))
		{
			short v;
			float f[3];
			
			fscanf(s, "%d", &v);
			fscanf(s, "%f %f %f", &f[0], &f[1], &f[2]);

			bfm_bin.WriteSig(SIG_VERTEX);
			bfm_bin.WriteFloat32((PBFMFLOAT32)f, 3);
			printf("* vtx %d: %f, %f, %f\n", v, f[0], f[1], f[2]);
		}
		// read face
		else if (!strcmp (data, FACE))
		{
			short v;
			short vv[3];
			
			fh.face_count++;
			
			fscanf(s, "%d:", &v);
			fscanf(s, " A:\t%d B:\t%d C:\t%d", &vv[0], &vv[1], &vv[2]);

			bfm_bin.WriteSig(SIG_FACE);
			bfm_bin.WriteInt16((PBFMINT16)vv, 3);
			printf("* face %d: %d, %d, %d\n", v, vv[0], vv[1], vv[2]);
		}
	}
}


void aseConv(char* fn)
{
	FILE *s;

	if (!(s = fopen(fn, "r")))
	{
		// file not found, bummer
		printf("# %s not found!\n", fn);
		return;
	}

	aseParse(s);

	fclose(s);
}

void main(int argc, char* argv[])
{
	printf("ASEC - ASE Converter\n");

	if (argc < 2)
		return;

	bfm_bin.Init("out.bin");
	bfm_bin.OpenToWrite();

	// save a room for file header information
	BFMINT32	savepos;
	bfm_bin.WriteCurrent(&savepos);
	bfm_bin.WriteSkip(sizeof(FILE_HEADER));

	aseConv(argv[1]);

	bfm_bin.WriteSeek(savepos);
	bfm_bin.WriteInt8((PBFMINT8)&fh, sizeof(FILE_HEADER));
}

